import requests
import jwt
from cryptography.hazmat.primitives.serialization import load_pem_public_key
from cryptography.hazmat.backends import default_backend
from typing import Callable
from urllib.parse import urlparse, parse_qs

class MiniOrange:
    def __init__(self, client_id: str, client_secret: str, base_url: str, redirect_url: str, public_key_pem: str):
        self.client_id = client_id
        self.client_secret = client_secret
        self.base_url = base_url
        self.redirect_url = redirect_url
        self.public_key = self.load_public_key(public_key_pem)

    @staticmethod
    def load_public_key(pem_data: str):
        try:
            public_key = load_pem_public_key(pem_data.encode(), backend=default_backend())
            return public_key
        except ValueError as e:
            raise ValueError(f"Error loading public key: {e}")

    def start_authorization(self):
        auth_url = (
            f"{self.base_url}/moas/idp/openidsso?"
            f"client_id={self.client_id}&"
            f"redirect_uri={self.redirect_url}&"
            f"scope=openid&"
            f"response_type=code&"
            f"state=yAwL-57K10sIIpGeVO7nR7ZAnzdsj01uGothExyVpmo"
        )
        return auth_url

    def handle_authorization_response(self, uri: str, callback: Callable[[str], None]):
        parsed_uri = urlparse(uri)
        if parsed_uri.path.startswith(f"{self.base_url}/lander"):
            query_params = parse_qs(parsed_uri.query)
            code = query_params.get("code", [None])[0]
            if code:
                self.request_token(code, callback)

    def request_token(self, code: str, callback: Callable[[str], None]):
        post_url = f"{self.base_url}/moas/rest/oauth/token"
        params = {
            "grant_type": "authorization_code",
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": self.redirect_url,
            "code": code
        }
        try:
            response = requests.post(post_url, data=params)
            response.raise_for_status()
            data = response.json()
            id_token = data.get("id_token")
            if id_token:
                callback(id_token)
            else:
                callback("ID token not found in response")
        except requests.RequestException as e:
            callback(f"Request error: {e}")

    def decode_jwt(self, id_token: str):
        try:
            decoded_payload = jwt.decode(
                id_token,
                self.public_key,
                algorithms=['RS256'],
                audience=self.client_id
            )
            return decoded_payload
        except jwt.ExpiredSignatureError:
            raise ValueError("ID token has expired.")
        except jwt.InvalidTokenError as e:
            raise ValueError(f"Invalid ID token: {e}")

    @staticmethod
    def fetch_user_info(url: str , token: str , callback: Callable[[str], None]):
        headers = {"Authorization": f"Bearer {token}"}
        try:
            response = requests.get(url, headers=headers)
            response.raise_for_status()
            callback(response.text)
        except requests.RequestException as e:
            callback(f"Request error: {e}")



import unittest
from unittest.mock import patch, Mock
from src import MiniOrange

class TestMiniOrange(unittest.TestCase):

    def setUp(self):
        """Set up the MiniOrange instance for testing."""
        self.mo = MiniOrange(context="test_context")
        self.mo.set_client_id("gOFOPctXqqAZzvU")
        self.mo.set_client_secret("2sgYyB8EOX-HpVi8TpHklUAXUVQ")
        self.mo.set_base_url("https://vk.xecurify.com")
        self.mo.set_redirect_url("http://127.0.0.1:5000/callback")

    @patch('src.requests.post')
    def test_request_token_success(self, mock_post):
        """Test successful token request."""
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"id_token": "mock_token"}
        mock_post.return_value = mock_response

        def callback(result):
            self.assertEqual(result, "mock_token")

        self.mo.request_token("mock_code", callback)

    @patch('src.requests.post')
    def test_request_token_failure(self, mock_post):
        """Test token request failure."""
        mock_response = Mock()
        mock_response.status_code = 400
        mock_response.json.return_value = {}
        mock_post.return_value = mock_response

        def callback(result):
            self.assertEqual(result, "ID token not found in response")

        self.mo.request_token("mock_code", callback)

    @patch('src.requests.get')
    def test_fetch_user_info_success(self, mock_get):
        """Test successful user info fetch."""
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.text = '{"email": "test@example.com"}'
        mock_get.return_value = mock_response

        def callback(result):
            self.assertEqual(result, '{"email": "test@example.com"}')

        self.mo.fetch_user_info("https://example.com/userinfo", "mock_token", callback)

    @patch('src.requests.get')
    def test_fetch_user_info_failure(self, mock_get):
        """Test failure in fetching user info."""
        mock_response = Mock()
        mock_response.status_code = 400
        mock_get.return_value = mock_response

        def callback(result):
            self.assertTrue(result.startswith("Request error"))

        self.mo.fetch_user_info("https://example.com/userinfo", "mock_token", callback)

    def test_start_authorization_missing_parameters(self):
        """Test start authorization when parameters are missing."""
        mo = MiniOrange(context="test_context")
        with patch('src.webbrowser.open') as mock_open:
            mo.start_authorization()
            mock_open.assert_not_called()

    @patch('src.webbrowser.open')
    def test_start_authorization_success(self, mock_open):
        """Test start authorization with complete parameters."""
        self.mo.start_authorization()
        expected_url = (
            'https://vk.xecurify.com/moas/idp/openidsso?'
            'client_id=gOFOPctXqqAZzvU&'
            'redirect_uri=http://127.0.0.1:5000/callback&'
            'scope=openid&'
            'response_type=code&'
            'state=yAwL-57K10sIIpGeVO7nR7ZAnzdsj01uGothExyVpmo'
        )
        mock_open.assert_called_once_with(expected_url)

    def test_handle_authorization_response_no_code(self):
        """Test handling of authorization response when no code is present."""
        def callback(result):
            self.assertEqual(result, "SSO configuration is incomplete")

        self.mo.handle_authorization_response("https://example.com/lander", callback)

if __name__ == "__main__":
    unittest.main()
